import {
  schemeCategory10,
  zoom,
  select,
  event,
  csv,
  scaleLinear,
  scaleOrdinal,
  extent
} from 'd3';

const {
  Scene,
  Object3D,
  PerspectiveCamera,
  WebGLRenderer,
  CanvasTexture,
  Sprite,
  SpriteMaterial,
  loadTexture,
  ImageUtils,
  PointLight,
  SpriteAlignment,
  shadowMap,
  BoxGeometry,

  
} = THREE;

// import SVG to Canvas .js for cleaner code.
import { ctx } from './pikaSVG';


// pikachue face background on canvas d3.js format, converted from svg to d3.js canvas format.
const canvas_svg = ctx;

// Tweakables
const scaleFactor = 2; // HiDPI
const width = window.innerWidth * scaleFactor;
const height = window.innerHeight * scaleFactor;
const rotationSensitivity = 0.04;
const rotationIncrement = -0.2;

// Returns a color for a given datum and number of clusters.
const scene = new Scene();
const camera = new PerspectiveCamera(50, width / height, 1, 3000);
const renderer = new WebGLRenderer({ antialias: true, alpha: true });


var controls = new THREE.OrbitControls(camera, renderer.domElement);

renderer.shadowMap.enabled = true;
renderer.shadowMap.type = THREE.BasicShadowMAp;
const ambientLight = new THREE.AmbientLight(0xffffff, 0.5);
scene.add(ambientLight);

renderer.setSize(width, height);
document.body.appendChild(renderer.domElement);
renderer.domElement.style.width = width / scaleFactor + 'px';
renderer.domElement.style.height = height / scaleFactor + 'px';

//Pika Dice
const geometry = new BoxGeometry(5, 5, 5);
const loader = new THREE.TextureLoader();
const texture = new THREE.CanvasTexture(ctx.canvas);
const material = new THREE.MeshPhongMaterial({ map: texture});
const cube = new THREE.Mesh(geometry, material);
cube.receiveShadow = true;
cube.castShadow = true;
scene.add(cube);

//GROUND
var groundTexture = new THREE.TextureLoader().load(
  'https://raw.githubusercontent.com/stemkoski/stemkoski.github.com/master/Three.js/images/checkerboard.jpg'
);
groundTexture.wrapS = groundTexture.wrapT = THREE.RepeatWrapping;
groundTexture.repeat.set(5, 5);
var groundMaterial = new THREE.MeshPhongMaterial({
  map: groundTexture,
  side: THREE.DoubleSide
});
var groundGeometry = new THREE.PlaneGeometry(100, 100, 10, 10);
var ground = new THREE.Mesh(groundGeometry, groundMaterial);
ground.receiveShadow = true;
scene.add(ground);

//LIGHT
const light = new THREE.PointLight(0xffffff, 1, 15);
light.position.set(-5, 10, 5);
light.castShadow = true;
light.shadow.camera.near = 0.3;
light.shadow.camera.far = 25;
scene.add(light);

//Init ground roration
ground.rotation.x -= Math.PI / 2;

//Init cube position
cube.position.z = 0;
cube.position.x = 0;
cube.position.y = 4;

//Init camera position
camera.position.set(5, 15, 15);

// Set up rotation interaction.
const rotationInteraction = zoom();

// Disable zoom, only use panning.
rotationInteraction.scaleExtent([1, 1]);

// Limit panning so it can't go beyond the north and south poles.
// Without this, moving the mouse right and left could cause
// undesirable inverse rotation.
rotationInteraction.translateExtent([
	[-Infinity, -Math.PI / 2 / rotationSensitivity],
  [Infinity, height + Math.PI / 2 / rotationSensitivity]
]);

// Update scenegraph root rotation on pan.
rotationInteraction.on('zoom', () => {
  cube.rotation.y = event.transform.x * rotationSensitivity;
	cube.rotation.x = event.transform.y * rotationSensitivity;
});

// Set up the interaction to respond to the DOM events.
const rotationSelection = select(renderer.domElement).call(rotationInteraction);

// Initialize rotation
rotationInteraction.translateBy(rotationSelection, 0, 0);
  

// Rotate slowly all the time.
const animate = () => {
requestAnimationFrame(animate);
controls.update();
 var xSpeed = 0.1;
var ySpeed = 0.1;

document.addEventListener('keypress', onDocumentKeyPress, false);
function onDocumentKeyPress(event) {
    var keyCode = event.which;
  	console.log(keyCode);
    if (keyCode == 87) {
        cube.position.y += ySpeed;
    } else if (keyCode == 83) {
        cube.position.y -= ySpeed;
    } else if (keyCode == 65) {
        cube.position.x -= xSpeed;
    } else if (keyCode == 68) {
        cube.position.x += xSpeed;
    } else if (keyCode == 32) {
        cube.position.set(0, 0, 0);
    }
}
  
  
 	rotationInteraction.translateBy(rotationSelection, rotationIncrement, 0);
  
  
  renderer.render(scene, camera);
};
animate();
